/*

 # Editor : Holiday
 # Date   : 20160117
 # Product: NFC Module for Arduino
 # SKU    : DFR0231
 
 Product Link 
 https://item.taobao.com/item.htm?spm=a1z10.3-c.w4002-3141887416.9.fNDjXd&id=18882986346
 
 PN532 reads the tag by Leonardo

*/

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#define print1Byte(args) Serial1.write(args)
#define print1lnByte(args)  Serial1.write(args),Serial1.println()
#else
#include "WProgram.h"
#define print1Byte(args) Serial1.print(args,BYTE)
#define print1lnByte(args)  Serial1.println(args,BYTE)
#endif

unsigned char receive_ACK[35];
unsigned char UID[4]={0xD1,0xAA,0x40,0xEA};
unsigned char secret[6]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
unsigned char dataWriteIntoCard[16]={0x01,0x01,0x02,0x02,0x03,0x03,0x04,0x04,0x05,0x05,0x06,0x06,0x07,0x07,0x08,0x08};

int ctr=0;

void setup()
{
  Serial.begin(9600);
  Serial1.begin(115200);

  wakeUp();
  delay(10);
  readAck(15);
  for(int i=0;i<15;i++)  Serial.print(receive_ACK[i]);
  Serial.println();

}

void loop()
{ 
  Scan();
  if(passWordCheck(0x08,UID,secret)==1)
 {   
   Serial.println("passed");   
  if(ctr<4)
  {  
	writeData(0x08,dataWriteIntoCard);
	Serial.println("written");
	ctr++;
  } 
  delay(2000);
  readData(0x08);    
  } 
  delay(4000);
}

void wakeUp()
{
  const unsigned char wake[24]={
  0x55, 0x55, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, \
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0x03, 0xfd, 0xd4, 0x14, 0x01, 0x17, 0x00};//wake up NFC module
  for(int i=0;i<24;i++) //send command
  {
    UART1_Send_Byte(wake[i]);
  }
}

void Scan()
{
  const unsigned char cmdUID[11]={ 0x00, 0x00, 0xFF, 0x04, 0xFC, 0xD4, 0x4A, 0x01, 0x00, 0xE1, 0x00};
  //0 0 FF 0 FF 0 0 0 FF C F4 D5 4B 1 1 0 4 8 4 D1 AA 40 EA 29 0 
  //0 0 FF 0 FF 0 ----ACK
  //0 0 FF C F4 
  //D5---PN532 to Arduino
  //4B----respond command 
  //1 1----target ID  and target amount
  //0 4----atq 
  //8----capacity of the card  is 8K
  //4 ---- 4 numbers of the UID
  //D1 AA 40 EA----UID 
  //29 0------DCS  POST---  DCS=0xff&(SUM(0 0 FF C F4 D5 4B 1 1 0 4 8 4 D1 AA 40 EA))
  unsigned char NFC_UID[5],count=0; 
  for(int i=0;i<11;i++)  UART1_Send_Byte(cmdUID[i]);
  delay(10); 
  readAck(25);
  delay(10); 
}


int passWordCheck(int block,unsigned char id[],unsigned char st[])
{
  //---------head-------  card 1  check blocknumber  password               UID D1 AA 40 EA      DCS+POST
  //00 00 FF 0F F1 D4 40    01     60    07          FF FF FF FF FF FF       02 F5 13 BE           C2 00
  
  //out: 00 00 FF 00 FF 00 00 00 FF 03 FD D5 41 00 EA 00  ------41 00 //ȷ״̬    16Bytes
  //     0  0  FF 0  FF 0  0  0  FF  3 FD D5 41 27 C3 0 
  unsigned char cmdPassWord[22]={0x00,0x00,0xFF,0x0F,0xF1,0xD4,0x40,0x01,0x60,\
                                 0x07,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xD1,0xAA,0x40,0xEA,0xC2,0x00};                                 
  unsigned char sum=0,count=0;
  cmdPassWord[9]=block;
  for(int i=10;i<16;i++) cmdPassWord[i]=st[i-10];// 
  for(int i=16;i<20;i++) cmdPassWord[i]=UID[i-16];// UID
  for(int i=0;i<20;i++) sum+=cmdPassWord[i];
  cmdPassWord[20]=0xff-sum&0xff;
  
  Serial.println("passWordSend: ");
  for(int i=0;i<22;i++)  {Serial.print(cmdPassWord[i],HEX); Serial.print(" ");}
  Serial.println();
  
 
  while(Serial1.available())   char xx=Serial1.read();//clear the serial data
  
  for (int i=0;i<22;i++)  UART1_Send_Byte(cmdPassWord[i]);
  delay(100);
  while(Serial1.available())
     {
        receive_ACK[count]=Serial1.read();
        count++;
     }
 // readAck(16);delay(10);
  Serial.println("passWordRes: ");
  for(int i=0;i<16;i++)  {Serial.print(receive_ACK[i],HEX); Serial.print(" ");}
  Serial.println();
  
  if(checkDCS(16)==1 && receive_ACK[12]==0x41 && receive_ACK[13]==0x00)  return 1;
  else return 0; 
}

void readData(int block)// ȡ  block---ҪȡĿ
{
  //----head--------- cmd  card 1   readcmd   block 07  DCS+POST
  //00 00 ff 05 fb D4  40    01      30        07        B4 00 //7
  //out: 00 00 FF 00 FF 00 //ACK
  //------------------41 00for right  41 03wrong        data                                           DCS+POST
  //00 00 FF 13 ED D5 41 00                        00 00 00 00 00 00 FF 07 80 69 FF FF FF FF FF FF     01 00  //7

  //test back:00 00 FF 00 FF 00 00 00 FF 13 ED D5 41 00 01 01 02 02 03 03 04 04 05 05 06 06 07 07 08 08 A2 00 
  unsigned char cmdRead[12]={0x00,0x00,0xff,0x05,0xfb,0xD4,0x40,0x01,0x30,0x07,0xB4,0x00};
  unsigned char sum=0,count=0;
  cmdRead[9]=block;
  for(int i=0;i<10;i++) sum+=cmdRead[i];
  cmdRead[10]=0xff-sum&0xff;
  
  while(Serial1.available())   char xx=Serial1.read();//clear the serial data
  
  //Serial.print("readDCS:");Serial.println(cmdRead[10],HEX);
  for(int i=0;i<12;i++){UART1_Send_Byte(cmdRead[i]);}
  delay(10);
  while(Serial1.available())
     {
        receive_ACK[count]=Serial1.read();
////        Serial.print(count);Serial.print(":"); 
//        Serial.print(receive_ACK[count],HEX);
//        Serial.print(" ");
        count++;
     }
     
  Serial.println("Read data:   ");
  for(int i=0;i<count;i++)  {Serial.print(receive_ACK[i],HEX); Serial.print(" ");}
  Serial.println();
  //    DCSУǷɹ        λΪ0x410x00ʾɹ
  if(checkDCS(32)==1 && receive_ACK[12]==0x41 && receive_ACK[13]==0x00)  Serial.println("Finish Reading");
  
  Serial.println(" ");
}

void writeData(int block,unsigned char dwic[])//  blockдݵĿ飬dwic[]д
{
  //00 00 ff 15 EB D4 40 01 A0 06 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F CD 00
  unsigned char cmdWrite[]={0x00,0x00,0xff,0x15,0xEB,0xD4,0x40,0x01,0xA0, \
                            0x06,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07, \
                            0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,0xCD,0x00};
  unsigned char sum=0,count=0;
  cmdWrite[9]=block;
  for(int i=10;i<26;i++) cmdWrite[i]=dwic[i-10];// д
  for(int i=0;i<26;i++) sum+=cmdWrite[i];//Ӻ
  cmdWrite[26]=0xff-sum&0xff;//  DCS
  
  while(Serial1.available())   char xx=Serial1.read();//clear the serial data
  
  for(int i=0;i<28;i++) UART1_Send_Byte(cmdWrite[i]);
  while(Serial1.available())
     {
        receive_ACK[count]=Serial1.read();
        count++;
     }
     
  Serial.print("Write respond:   ");
  for(int i=0;i<17;i++)  {Serial.print(receive_ACK[i],HEX); Serial.print(" ");}
  Serial.println("     Write respond End ");
  if(checkDCS(17)==1 && receive_ACK[13]==0x41 && receive_ACK[14]==0x00)
  {
    Serial.println("WriteFinish!");
  }
}

void readAck(int x) //ȡxڷ
{
  unsigned char i;
  for(i=0;i<x;i++)
  {
    receive_ACK[i]= Serial1.read();
  }
}

void UART1_Send_Byte(unsigned char command_data)  // չ̶ʽ
{
  print1Byte(command_data);
  #if defined(ARDUINO) && ARDUINO >= 100
    Serial1.flush();// complete the transmission of outgoing serial data 
  #endif
} 

char checkDCS(int x)  //  NFC  S50  DCSУӺ
{
  unsigned char sum=0,dcs=0;
  for(int i=6;i<x-2;i++)
  {
    sum+=receive_ACK[i];
  }
  dcs=0xff-sum&0xff;
  if(dcs==receive_ACK[x-2])  return 1;
  else   return 0;
}

